/*
 * Copyright 2009 OW2 Chameleon
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.chameleon.json.test;

import java.text.ParseException;
import java.util.List;
import java.util.Map;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.ow2.chameleon.json.impl.JSONServiceProvider;


public class FromStringTest extends TestCase {

    public static final String SIMPLE_JSON = "{foo='foo'}";

    public static final String SIMPLE_JSON_2 = "{foo='foo', bar='bar'}";

    public static final String SIMPLE_JSON_3 = "{"
            + "foo='foo', one=1, true=true, false=false, nil=null, array=[1,2,3], array2=[[1,2],[{foo='foo'}]], double=2.4, object="
            + SIMPLE_JSON_2 + "}";

    public static final String EMPTY_JSON = "{}";

    public static final String EMPTY = "";

    public static final String COLLECTION = "{items=[{x=1, y=1}, {x=2, y=2}, {x=3, y=3}]}";

    public static final String STORE = "{"
            + "\"identifier\":\"id\","
            + "\"label\":\"id\","
            + "\"items\":["
            + "{\"id\":\"dd\",\"funktion\":\"dd\",\"gangzahl\":\"dd\",\"sling:resourceType\":\"bwb:schieber\",\"nennweite\":\"dd\",\"typ\":\"dd\",\"stellmoment\":\"dd\",\"adresse\":\"dd\",\"jcr:primaryType\":\"nt:unstructured\",\"baujahr\":\"dd\"},"
            + "{\"id\":\"dd2\",\"sling:resourceType\":\"bwb:schieber\",\"gangzahl\":\"ww\",\"funktion\":\"ggg\",\"nennweite\":\"eee\",\"typ\":\"aa\",\"adresse\":\"vv\",\"stellmoment\":\"qqq\",\"baujahr\":\"jjj\",\"jcr:primaryType\":\"nt:unstructured\"},"
            + "]}";

    JSONServiceProvider svc;

    public void setUp() {
        svc = new JSONServiceProvider();
    }

    public void testFromJSON() throws ParseException {
        Map m1 = svc.fromJSON(SIMPLE_JSON);
        Assert.assertEquals("foo", m1.get("foo"));
        Assert.assertEquals(1, m1.size());

        Map m2 = svc.fromJSON(SIMPLE_JSON_2);
        Assert.assertEquals("foo", m2.get("foo"));
        Assert.assertEquals("bar", m2.get("bar"));
        Assert.assertEquals(2, m2.size());

        Map m3 = svc.fromJSON("{}");
        Assert.assertEquals(0, m3.size());

        Map m4 = svc.fromJSON(EMPTY);
        Assert.assertEquals(0, m4.size());

        Map m5 = svc.fromJSON((String) null);
        Assert.assertEquals(0, m5.size());

        Map m6 = svc.fromJSON(SIMPLE_JSON_3);
        Assert.assertEquals(new Integer(1), m6.get("one"));
        Assert.assertEquals(new Double(2.4), m6.get("double"));
        Assert.assertEquals(Boolean.TRUE, m6.get("true"));
        Assert.assertEquals(Boolean.FALSE, m6.get("false"));
        Assert.assertEquals("foo", m6.get("foo"));
        Assert.assertEquals(svc.getNull(), m6.get("nil"));
        Assert.assertFalse(null == m6.get("nil")); // It's not null it's NULL
                                                   // (Javascript) !
        Map m62 = (Map) m6.get("object");
        Assert.assertEquals("foo", m62.get("foo"));
        Assert.assertEquals("bar", m62.get("bar"));
        Assert.assertEquals(2, m62.size());
        List list = (List) m6.get("array");
        Assert.assertEquals(3, list.size());
        Assert.assertEquals(Integer.class.getName(), list.get(0).getClass()
                .getName());
    }

    public void testFromJSONWithArray() throws ParseException {
        Map m1 = svc.fromJSON(COLLECTION);
        Assert.assertEquals(1, m1.size());
        List list = (List) m1.get("items");
        Assert.assertEquals(3, list.size());
        Map m11 = (Map) list.get(0);
        Assert.assertEquals(new Integer(1), m11.get("x"));
        Assert.assertEquals(new Integer(1), m11.get("y"));
    }

    public void testFromJSONOnStore() throws ParseException {
        Map m1 = svc.fromJSON(STORE);
        Assert.assertEquals(3, m1.size());
        List list = (List) m1.get("items");
        Assert.assertEquals(2, list.size());
        Map m11 = (Map) list.get(0);
        Assert.assertEquals("dd", m11.get("funktion"));
        Assert.assertEquals("dd", m11.get("nennweite"));
    }

    public void testErrorInSimpleObject() {
        try {
            svc.fromJSON("{xx=xx, y.y=2,2}");
        } catch (ParseException e) {
            // OK
            return;
        }
        fail("ParseException expected");
    }

    public void testErrorInArray() throws ParseException {
        try {
            svc.fromJSON("{xx=[{foo=foo}, {bar=bar]}");
        } catch (ParseException e) {
            // OK
            return;
        }
        fail("ParseException expected");
    }
}
